@@ -13,7 +13,8 @@
 
 #include "json.h"
 
-static int showindices = 0; /* -n flag: show indices count for arrays */
+static int nflag = 0; /* -n flag: show indices count for arrays */
+static int rflag = 0; /* -r flag: show all control-characters */
 
 void
 printvalue(const char *s)
@@ -26,7 +27,7 @@ printvalue(const char *s)
 		case '\t': putchar('\\'); putchar('t'); break;
 		default:
 			/* ignore other control chars */
-			if (iscntrl((unsigned char)*s))
+			if (!rflag && iscntrl((unsigned char)*s))
 				continue;
 			putchar(*s);
 		}
@@ -48,7 +49,7 @@ processnode(struct json_node *nodes, size_t depth, const char *value)
 		if (nodes[i].type == TYPE_OBJECT) {
 			putchar('.');
 		} else if (nodes[i].type == TYPE_ARRAY) {
-			if (showindices) {
+			if (nflag) {
 				printf("[%zu]", nodes[i].index);
 			} else {
 				putchar('[');
@@ -67,13 +68,23 @@ processnode(struct json_node *nodes, size_t depth, const char *value)
 int
 main(int argc, char *argv[])
 {
+	int i, j;
+
 	if (pledge("stdio", NULL) == -1) {
 		fprintf(stderr, "pledge stdio: %s\n", strerror(errno));
 		return 1;
 	}
 
-	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'n')
-		showindices = 1;
+	for (i = 1; i < argc; i++) {
+		if (argv[i][0] != '-')
+			continue;
+		for (j = 1; argv[i][j]; j++) {
+			switch (argv[i][j]) {
+			case 'n': nflag = 1; break;
+			case 'r': rflag = 1; break;
+			}
+		}
+	}
 
 	switch (parsejson(processnode)) {
 	case JSON_ERROR_MEM:
